/***************************************************************************\
* Module Name: KbFrVmu.C
*
* keyboard layout
*
* Copyright (c) 1985-2001, Microsoft Corporation
*
* History:
* KBDTOOL v3.40 - Created  Thu Apr 26 11:03:30 2007
\***************************************************************************/

#include <windows.h>
#include "kbd.h"
#include "KbFrVmu.h"

#if defined(_M_IA64)
#pragma section(".data")
#define ALLOC_SECTION_LDATA __declspec(allocate(".data"))
#else
#pragma data_seg(".data")
#define ALLOC_SECTION_LDATA
#endif

/***************************************************************************\
* ausVK[] - Virtual Scan Code to Virtual Key conversion table
\***************************************************************************/

static ALLOC_SECTION_LDATA USHORT ausVK[] = {
    T00, T01, T02, T03, T04, T05, T06, T07,
    T08, T09, T0A, T0B, T0C, T0D, T0E, T0F,
    T10, T11, T12, T13, T14, T15, T16, T17,
    T18, T19, T1A, T1B, T1C, T1D, T1E, T1F,
    T20, T21, T22, T23, T24, T25, T26, T27,
    T28, T29, T2A, T2B, T2C, T2D, T2E, T2F,
    T30, T31, T32, T33, T34, T35,

    /*
     * Right-hand Shift key must have KBDEXT bit set.
     */
    T36 | KBDEXT,

    T37 | KBDMULTIVK,               // numpad_* + Shift/Alt -> SnapShot

    T38, T39, T3A, T3B, T3C, T3D, T3E,
    T3F, T40, T41, T42, T43, T44,

    /*
     * NumLock Key:
     *     KBDEXT     - VK_NUMLOCK is an Extended key
     *     KBDMULTIVK - VK_NUMLOCK or VK_PAUSE (without or with CTRL)
     */
    T45 | KBDEXT | KBDMULTIVK,

    T46 | KBDMULTIVK,

    /*
     * Number Pad keys:
     *     KBDNUMPAD  - digits 0-9 and decimal point.
     *     KBDSPECIAL - require special processing by Windows
     */
    T47 | KBDNUMPAD | KBDSPECIAL,   // Numpad 7 (Home)
    T48 | KBDNUMPAD | KBDSPECIAL,   // Numpad 8 (Up),
    T49 | KBDNUMPAD | KBDSPECIAL,   // Numpad 9 (PgUp),
    T4A,
    T4B | KBDNUMPAD | KBDSPECIAL,   // Numpad 4 (Left),
    T4C | KBDNUMPAD | KBDSPECIAL,   // Numpad 5 (Clear),
    T4D | KBDNUMPAD | KBDSPECIAL,   // Numpad 6 (Right),
    T4E,
    T4F | KBDNUMPAD | KBDSPECIAL,   // Numpad 1 (End),
    T50 | KBDNUMPAD | KBDSPECIAL,   // Numpad 2 (Down),
    T51 | KBDNUMPAD | KBDSPECIAL,   // Numpad 3 (PgDn),
    T52 | KBDNUMPAD | KBDSPECIAL,   // Numpad 0 (Ins),
    T53 | KBDNUMPAD | KBDSPECIAL,   // Numpad . (Del),

    T54, T55, T56, T57, T58, T59, T5A, T5B,
    T5C, T5D, T5E, T5F, T60, T61, T62, T63,
    T64, T65, T66, T67, T68, T69, T6A, T6B,
    T6C, T6D, T6E, T6F, T70, T71, T72, T73,
    T74, T75, T76, T77, T78, T79, T7A, T7B,
    T7C, T7D, T7E

};

static ALLOC_SECTION_LDATA VSC_VK aE0VscToVk[] = {
        { 0x10, X10 | KBDEXT              },  // Speedracer: Previous Track
        { 0x19, X19 | KBDEXT              },  // Speedracer: Next Track
        { 0x1D, X1D | KBDEXT              },  // RControl
        { 0x20, X20 | KBDEXT              },  // Speedracer: Volume Mute
        { 0x21, X21 | KBDEXT              },  // Speedracer: Launch App 2
        { 0x22, X22 | KBDEXT              },  // Speedracer: Media Play/Pause
        { 0x24, X24 | KBDEXT              },  // Speedracer: Media Stop
        { 0x2E, X2E | KBDEXT              },  // Speedracer: Volume Down
        { 0x30, X30 | KBDEXT              },  // Speedracer: Volume Up
        { 0x32, X32 | KBDEXT              },  // Speedracer: Browser Home
        { 0x35, X35 | KBDEXT              },  // Numpad Divide
        { 0x37, X37 | KBDEXT              },  // Snapshot
        { 0x38, X38 | KBDEXT              },  // RMenu
        { 0x47, X47 | KBDEXT              },  // Home
        { 0x48, X48 | KBDEXT              },  // Up
        { 0x49, X49 | KBDEXT              },  // Prior
        { 0x4B, X4B | KBDEXT              },  // Left
        { 0x4D, X4D | KBDEXT              },  // Right
        { 0x4F, X4F | KBDEXT              },  // End
        { 0x50, X50 | KBDEXT              },  // Down
        { 0x51, X51 | KBDEXT              },  // Next
        { 0x52, X52 | KBDEXT              },  // Insert
        { 0x53, X53 | KBDEXT              },  // Delete
        { 0x5B, X5B | KBDEXT              },  // Left Win
        { 0x5C, X5C | KBDEXT              },  // Right Win
        { 0x5D, X5D | KBDEXT              },  // Application
        { 0x5F, X5F | KBDEXT              },  // Speedracer: Sleep
        { 0x65, X65 | KBDEXT              },  // Speedracer: Browser Search
        { 0x66, X66 | KBDEXT              },  // Speedracer: Browser Favorites
        { 0x67, X67 | KBDEXT              },  // Speedracer: Browser Refresh
        { 0x68, X68 | KBDEXT              },  // Speedracer: Browser Stop
        { 0x69, X69 | KBDEXT              },  // Speedracer: Browser Forward
        { 0x6A, X6A | KBDEXT              },  // Speedracer: Browser Back
        { 0x6B, X6B | KBDEXT              },  // Speedracer: Launch App 1
        { 0x6C, X6C | KBDEXT              },  // Speedracer: Launch Mail
        { 0x6D, X6D | KBDEXT              },  // Speedracer: Launch Media Selector
        { 0x1C, X1C | KBDEXT              },  // Numpad Enter
        { 0x46, X46 | KBDEXT              },  // Break (Ctrl + Pause)
        { 0,      0                       }
};

static ALLOC_SECTION_LDATA VSC_VK aE1VscToVk[] = {
        { 0x1D, Y1D                       },  // Pause
        { 0   ,   0                       }
};

/***************************************************************************\
* aVkToBits[]  - map Virtual Keys to Modifier Bits
*
* See kbd.h for a full description.
*
* The keyboard has only three shifter keys:
*     SHIFT (L & R) affects alphabnumeric keys,
*     CTRL  (L & R) is used to generate control characters
*     ALT   (L & R) used for generating characters by number with numpad
\***************************************************************************/
static ALLOC_SECTION_LDATA VK_TO_BIT aVkToBits[] = {
    { VK_SHIFT    ,   KBDSHIFT     },
    { VK_CONTROL  ,   KBDCTRL      },
    { VK_MENU     ,   KBDALT       },
    { VK_OEM_8    ,   KBDKANA      },
    { 0           ,   0            }
};

/***************************************************************************\
* aModification[]  - map character modifier bits to modification number
*
* See kbd.h for a full description.
*
\***************************************************************************/

static ALLOC_SECTION_LDATA MODIFIERS CharModifiers = {
    &aVkToBits[0],
    9,
    {
    //  Modification# //  Keys Pressed
    //  ============= // =============
        0,            // 
        1,            // Shift 
        2,            // Control 
        SHFT_INVALID, // Shift + Control 
        SHFT_INVALID, // Alt 
        SHFT_INVALID, // Shift + Alt 
        3,            // Alt + ctrl
        4,            // Alt + ctrl + shift
        5,            // kana
        6             // kana + shift
     }
};

/***************************************************************************\
*
* aVkToWch2[]  - Virtual Key to WCHAR translation for 2 shift states
* aVkToWch3[]  - Virtual Key to WCHAR translation for 3 shift states
* aVkToWch4[]  - Virtual Key to WCHAR translation for 4 shift states
* aVkToWch5[]  - Virtual Key to WCHAR translation for 5 shift states
* aVkToWch6[]  - Virtual Key to WCHAR translation for 6 shift states
*
* Table attributes: Unordered Scan, null-terminated
*
* Search this table for an entry with a matching Virtual Key to find the
* corresponding unshifted and shifted WCHAR characters.
*
* Special values for VirtualKey (column 1)
*     0xff          - dead chars for the previous entry
*     0             - terminate the list
*
* Special values for Attributes (column 2)
*     CAPLOK bit    - CAPS-LOCK affect this key like SHIFT
*
* Special values for wch[*] (column 3 & 4)
*     WCH_NONE      - No character
*     WCH_DEAD      - Dead Key (diaresis) or invalid (US keyboard has none)
*     WCH_LGTR      - Ligature (generates multiple characters)
*
\***************************************************************************/

static ALLOC_SECTION_LDATA VK_TO_WCHARS2 aVkToWch2[] = {
//                      |         |  Shift  |
//                      |=========|=========|
  {VK_TAB       ,0      ,'\t'     ,'\t'     },
//  {VK_ADD       ,0      ,'+'      ,'+'      },
//  {VK_DIVIDE    ,0      ,'/'      ,'/'      },
//  {VK_MULTIPLY  ,0      ,'*'      ,'*'      },
//  {VK_SUBTRACT  ,0      ,'-'      ,'-'      },
  {0            ,0      ,0        ,0        }
};

static ALLOC_SECTION_LDATA VK_TO_WCHARS3 aVkToWch3[] = {
//                      |         |  Shift  |  Ctrl   |
//                      |=========|=========|=========|
  {VK_BACK      ,0      ,'\b'     ,'\b'     ,0x007f   },
  {VK_ESCAPE    ,0      ,0x001b   ,0x001b   ,0x001b   },
  {VK_RETURN    ,0      ,'\r'     ,'\r'     ,'\n'     },
  {VK_CANCEL    ,0      ,0x0003   ,0x0003   ,0x0003   },
  {0            ,0      ,0        ,0        ,0        }
};

static ALLOC_SECTION_LDATA VK_TO_WCHARS7 aVkToWch7[] = {
//                |         |  Shift  |  Ctrl   |  AltGr  | S+AltGr |  Kana   | S+Kana  |
//                |=========|=========|=========|=========|=========|=========|=========|
{'1'    ,0        ,'&'      ,'1'      ,WCH_NONE ,WCH_DEAD ,WCH_NONE ,0x2018   ,WCH_NONE },
{0xff   ,0        ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x00b4   ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{'2'    ,SGCAPS   ,0x00e9   ,'2'      ,WCH_NONE ,WCH_DEAD ,WCH_NONE ,0x2019   ,WCH_NONE },
{0xff   ,0        ,0x00c9   ,'2'      ,WCH_NONE ,'~'      ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{'3'    ,0        ,'\"'     ,'3'      ,WCH_NONE ,'#'      ,WCH_NONE ,0x201a   ,0x2020   },
{'4'    ,0        ,'\''     ,'4'      ,WCH_NONE ,'{'      ,WCH_NONE ,0x201b   ,0x2021   },
{'5'    ,0        ,'('      ,'5'      ,WCH_NONE ,'['      ,WCH_NONE ,0x201c   ,0x00b6   },
{'6'    ,0        ,'-'      ,'6'      ,WCH_NONE ,'|'      ,WCH_NONE ,0x201d   ,0x00a6   },
{'7'    ,SGCAPS   ,0x00e8   ,'7'      ,WCH_NONE ,WCH_DEAD ,WCH_NONE ,0x201e   ,WCH_NONE },
{0xff   ,0        ,0x00c8   ,'7'      ,WCH_NONE ,'`'      ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{'8'    ,0        ,'_'      ,'8'      ,WCH_NONE ,'\\'     ,WCH_NONE ,0x201f   ,0x00ad   },
{'9'    ,SGCAPS   ,0x00e7   ,'9'      ,WCH_NONE ,'^'      ,WCH_NONE ,0x00ab   ,0x2039   },
{'9'    ,0        ,0x00c7   ,'9'      ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{'0'    ,SGCAPS   ,0x00e0   ,'0'      ,WCH_NONE ,'@'      ,WCH_NONE ,0x00bb   ,0x203a   },
{'0'    ,0        ,0x00c0   ,'0'      ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{VK_OEM_4     ,0  ,')'      ,0x00b0   ,WCH_NONE ,']'      ,WCH_NONE ,0x2013   ,WCH_NONE },
{VK_OEM_PLUS  ,0  ,'='      ,'+'      ,WCH_NONE ,'}'      ,WCH_NONE ,0x2014   ,WCH_NONE },
{'A' ,CAPLOK | CAPLOKALTGR  ,'a' ,'A' ,WCH_NONE ,0x00e6   ,0x00c6   ,0xfb00   ,WCH_NONE },
{'Z' ,CAPLOK      ,'z'      ,'Z'      ,WCH_NONE ,0x00fe   ,0x00de   ,0xfb01   ,WCH_NONE },
{'E' ,CAPLOK      ,'e'      ,'E'      ,WCH_NONE ,0x20ac   ,WCH_NONE ,0xfb02   ,WCH_NONE },
{'R' ,CAPLOK      ,'r'      ,'R'      ,WCH_NONE ,WCH_DEAD ,WCH_NONE ,0x00ae   ,WCH_NONE },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x02da   ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{'T' ,CAPLOK | CAPLOKALTGR  ,'t' ,'T' ,WCH_NONE ,0x01ad   ,0x01ac   ,0x2122   ,WCH_NONE },
{'Y' ,CAPLOK | CAPLOKALTGR  ,'y' ,'Y' ,WCH_NONE ,0x01b4   ,0x01b3   ,0x00a5   ,WCH_NONE },
{'U' ,CAPLOK      ,'u'      ,'U'      ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0xfb03   ,WCH_NONE },
{'I' ,CAPLOK | CAPLOKALTGR  ,'i' ,'I' ,WCH_NONE ,0x00f8   ,0x00d8   ,0xfb04   ,WCH_NONE },
{'O' ,CAPLOK | CAPLOKALTGR  ,'o' ,'O' ,WCH_NONE ,0x0153   ,0x0152   ,WCH_NONE ,WCH_NONE },
{'P' ,CAPLOK | CAPLOKALTGR  ,'p' ,'P' ,WCH_NONE ,0x01a5   ,0x01a4   ,WCH_NONE ,WCH_NONE },
{VK_OEM_6 ,0      ,WCH_DEAD ,WCH_DEAD ,WCH_NONE ,WCH_DEAD ,WCH_DEAD ,0xfb05   ,WCH_NONE },
{0xff     ,0      ,0x02c6   ,0x00a8   ,WCH_NONE ,0x02cd   ,0x0311   ,WCH_NONE ,WCH_NONE },
{VK_OEM_1 ,0      ,'$'      ,0x00a3   ,WCH_NONE ,0x00a4   ,WCH_NONE ,0xfb06   ,WCH_NONE },
{'Q'    ,CAPLOK   ,'q'      ,'Q'      ,WCH_NONE ,0x2126   ,WCH_NONE ,WCH_NONE ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0xa708   },
{'S'    ,CAPLOK   ,'s'      ,'S'      ,WCH_NONE ,0x00df   ,0x017f   ,WCH_NONE ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0xa709   },
{'D' ,CAPLOK | CAPLOKALTGR  ,'d' ,'D' ,WCH_NONE ,0x00f0   ,0x00d0   ,WCH_NONE ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0xa70a   },
{'F' ,CAPLOK | CAPLOKALTGR  ,'f' ,'F' ,WCH_NONE ,0x0192   ,0x0191   ,0x00aa   ,WCH_DEAD },
{0xff      ,0     ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0xa70b   },
{'G' ,CAPLOK | CAPLOKALTGR  ,'g' ,'G' ,WCH_NONE ,0x0260   ,0x0193   ,WCH_NONE ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0xa70c   },
{'H'    ,CAPLOK   ,'h'      ,'H'      ,WCH_NONE ,0x0266   ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{'J' ,CAPLOK | CAPLOKALTGR  ,'j' ,'J' ,WCH_NONE ,0x0133   ,0x0132   ,WCH_NONE ,WCH_NONE },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{'K'    ,CAPLOK   ,'k'      ,'K'      ,WCH_NONE ,0x0199   ,0x0198   ,0x0138   ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0xa70d   },
{'L' ,CAPLOK | CAPLOKALTGR  ,'l' ,'L' ,WCH_NONE ,0x0140   ,0x013f   ,WCH_NONE ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0xa70e   },
{'M' ,CAPLOK      ,'m'      ,'M'      ,WCH_NONE ,0x0271   ,WCH_NONE ,0x00ba   ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0xa70f   },
{VK_OEM_3 ,SGCAPS ,0x00f9   ,'%'      ,WCH_NONE ,0x2030   ,WCH_NONE ,WCH_DEAD ,WCH_DEAD },
{VK_OEM_3 ,0      ,0x00d9   ,'%'      ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x02dd   ,0xa710   },
{VK_OEM_7 ,0      ,0x00b2   ,0x00b3   ,WCH_NONE ,0x00b9   ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{VK_OEM_5 ,0      ,'*'      ,0x00b5   ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_DEAD ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x030f   ,0xa711   },
{'W' ,CAPLOK | CAPLOKALTGR  ,'w' ,'W' ,WCH_NONE ,0x0142   ,0x0141   ,WCH_DEAD ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x02d8   ,0xa713   },
{'X' ,CAPLOK      ,'x'      ,'X'      ,WCH_NONE ,0x0257   ,0x018a   ,WCH_DEAD ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x02c0   ,0xa714   },
{'C' ,CAPLOK | CAPLOKALTGR  ,'c' ,'C' ,WCH_NONE ,0x0188   ,0x0187   ,0x00a9   ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x02db   ,0xa715   },
{'V' ,CAPLOK | CAPLOKALTGR  ,'v' ,'V' ,WCH_NONE ,0x028b   ,0x01b2   ,WCH_DEAD ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x02c7   ,0xa716   },
{'B' ,CAPLOK | CAPLOKALTGR ,'b'  ,'B' ,WCH_NONE ,0x0253   ,0x0181   ,WCH_DEAD ,WCH_NONE },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x0336   ,WCH_NONE },
{'N' ,CAPLOK | CAPLOKALTGR ,'n'  ,'N' ,WCH_NONE ,0x014b   ,0x014a   ,WCH_DEAD ,WCH_NONE },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x02c9   ,WCH_NONE },
{VK_OEM_COMMA ,0  ,','      ,'?'      ,WCH_NONE ,0x00bf   ,WCH_NONE ,WCH_DEAD ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x00b8   ,0xa706   },
{VK_OEM_PERIOD,0  ,';'      ,'.'      ,WCH_NONE ,0x00b7   ,WCH_NONE ,WCH_DEAD ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x0323   ,0xa704   },
{VK_OEM_2     ,0  ,':'      ,'/'      ,WCH_NONE ,0x2026   ,WCH_NONE ,WCH_DEAD ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x02db   ,0xa707   },
{VK_OEM_ATTN  ,0  ,'!'      ,0x00a7   ,WCH_NONE ,0x00a1   ,WCH_NONE ,WCH_DEAD ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x02d9   ,0xa705   },
{VK_SPACE     ,0  ,' '      ,' '      ,WCH_NONE ,0x00a0   ,0x00a0   ,WCH_NONE ,WCH_NONE },
{VK_OEM_102   ,0  ,'<'      ,'>'      ,WCH_NONE ,0x2264   ,0x2265   ,WCH_DEAD ,WCH_DEAD },
{0xff         ,0  ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x02bc   ,0xa712   },
{VK_DECIMAL   ,0  ,','      ,','      ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{VK_ADD       ,0  ,'+'      ,'+'      ,WCH_NONE ,0x00b1   ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{VK_DIVIDE    ,0  ,'/'      ,'/'      ,WCH_NONE ,0x00f7   ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{VK_MULTIPLY  ,0  ,'*'      ,'*'      ,WCH_NONE ,0x00d7   ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{VK_SUBTRACT  ,0  ,'-'      ,'-'      ,WCH_NONE ,0x00ac   ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{VK_NUMPAD1   ,0  ,'1'      ,WCH_NONE ,WCH_NONE ,0x00bc   ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{VK_NUMPAD2   ,0  ,'2'      ,WCH_NONE ,WCH_NONE ,0x00bd   ,WCH_NONE ,0x2193   ,WCH_NONE },
{VK_NUMPAD3   ,0  ,'3'      ,WCH_NONE ,WCH_NONE ,0x00be   ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{VK_NUMPAD4   ,0  ,'4'      ,WCH_NONE ,WCH_NONE ,0x2153   ,WCH_NONE ,0x2190   ,WCH_NONE },
{VK_NUMPAD5   ,0  ,'5'      ,WCH_NONE ,WCH_NONE ,0x2154   ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{VK_NUMPAD6   ,0  ,'6'      ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x2192   ,WCH_NONE },
{VK_NUMPAD7   ,0  ,'7'      ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{VK_NUMPAD8   ,0  ,'8'      ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,0x2191   ,WCH_NONE },
{VK_NUMPAD9   ,0  ,'9'      ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE ,WCH_NONE },
{0            ,0  ,0        ,0        ,0        ,0        ,0        ,0        ,0        }
};

// Put this last so that VkKeyScan interprets number characters
// as coming from the main section of the kbd (aVkToWch2 and
// aVkToWch5) before considering the numpad (aVkToWch1).

static ALLOC_SECTION_LDATA VK_TO_WCHARS1 aVkToWch1[] = {
    { VK_NUMPAD0   , 0      ,  '0'   },
//    { VK_NUMPAD1   , 0      ,  '1'   },
//    { VK_NUMPAD2   , 0      ,  '2'   },
//    { VK_NUMPAD3   , 0      ,  '3'   },
//    { VK_NUMPAD4   , 0      ,  '4'   },
//    { VK_NUMPAD5   , 0      ,  '5'   },
//    { VK_NUMPAD6   , 0      ,  '6'   },
//    { VK_NUMPAD7   , 0      ,  '7'   },
//    { VK_NUMPAD8   , 0      ,  '8'   },
//    { VK_NUMPAD9   , 0      ,  '9'   },
    { 0            , 0      ,  '\0'  }
};

static ALLOC_SECTION_LDATA VK_TO_WCHAR_TABLE aVkToWcharTable[] = {
    {  (PVK_TO_WCHARS1)aVkToWch3, 3, sizeof(aVkToWch3[0]) },
    {  (PVK_TO_WCHARS1)aVkToWch7, 7, sizeof(aVkToWch7[0]) },
    {  (PVK_TO_WCHARS1)aVkToWch2, 2, sizeof(aVkToWch2[0]) },
    {  (PVK_TO_WCHARS1)aVkToWch1, 1, sizeof(aVkToWch1[0]) },
    {                       NULL, 0, 0                    },
};

/***************************************************************************\
* aKeyNames[], aKeyNamesExt[]  - Virtual Scancode to Key Name tables
*
* Table attributes: Ordered Scan (by scancode), null-terminated
*
* Only the names of Extended, NumPad, Dead and Non-Printable keys are here.
* (Keys producing printable characters are named by that character)
\***************************************************************************/

static ALLOC_SECTION_LDATA VSC_LPWSTR aKeyNames[] = {
    0x01,    L"Esc",
    0x0e,    L"Backspace",
    0x0f,    L"Tab",
    0x1c,    L"Enter",
    0x1d,    L"Ctrl",
    0x2a,    L"Shift",
    0x36,    L"Right Shift",
    0x37,    L"Num *",
    0x38,    L"Alt",
    0x39,    L"Space",
    0x3a,    L"Caps Lock",
    0x3b,    L"F1",
    0x3c,    L"F2",
    0x3d,    L"F3",
    0x3e,    L"F4",
    0x3f,    L"F5",
    0x40,    L"F6",
    0x41,    L"F7",
    0x42,    L"F8",
    0x43,    L"F9",
    0x44,    L"F10",
    0x45,    L"Pause",
    0x46,    L"Scroll Lock",
    0x47,    L"Num 7",
    0x48,    L"Num 8",
    0x49,    L"Num 9",
    0x4a,    L"Num -",
    0x4b,    L"Num 4",
    0x4c,    L"Num 5",
    0x4d,    L"Num 6",
    0x4e,    L"Num +",
    0x4f,    L"Num 1",
    0x50,    L"Num 2",
    0x51,    L"Num 3",
    0x52,    L"Num 0",
    0x53,    L"Num Del",
    0x54,    L"Sys Req",
    0x57,    L"F11",
    0x58,    L"F12",
    0x7c,    L"F13",
    0x7d,    L"F14",
    0x7e,    L"F15",
    0x7f,    L"F16",
    0x80,    L"F17",
    0x81,    L"F18",
    0x82,    L"F19",
    0x83,    L"F20",
    0x84,    L"F21",
    0x85,    L"F22",
    0x86,    L"F23",
    0x87,    L"F24",
    0   ,    NULL
};

static ALLOC_SECTION_LDATA VSC_LPWSTR aKeyNamesExt[] = {
    0x1c,    L"Num Enter",
    0x1d,    L"Right Ctrl",
    0x35,    L"Num /",
    0x37,    L"Prnt Scrn",
    0x38,    L"Right Alt",
    0x45,    L"Num Lock",
    0x46,    L"Break",
    0x47,    L"Home",
    0x48,    L"Up",
    0x49,    L"Page Up",
    0x4b,    L"Left",
    0x4d,    L"Right",
    0x4f,    L"End",
    0x50,    L"Down",
    0x51,    L"Page Down",
    0x52,    L"Insert",
    0x53,    L"Delete",
    0x54,    L"<00>",
    0x56,    L"Help",
    0x5b,    L"Left Windows",
    0x5c,    L"Right Windows",
    0x5d,    L"Application",
    0   ,    NULL
};

static ALLOC_SECTION_LDATA DEADKEY_LPWSTR aKeyNamesDead[] = {
    L"\x02c6"	L"Circonflexe",
    L"\x00a8"	L"Trma",   
    L"`"	L"Accent grave",
    L"~"	L"Tilde",
    L"\x00b4"	L"Accent aigu",   
    L"\x02c7"   L"Caron",
    L"\x02c9"   L"Macron",
    L"\x02d8"	L"Brve",
    L"\x0323"	L"Point souscrit",
    L"\x02bc"	L"Cornu",
    L"\x02c0"	L"Crochet en chef",
    L"\x02da"   L"Rond en chef",
    L"\x0336"   L"Barre",
    L"\x02dd"   L"Double accent aigu",
    L"\x030f"   L"Double accent grave",
    L"\x02db"   L"Ogonek",
    L"\x02cd"   L"Macron souscrit",
    L"\x00b8"   L"Cdille",
    L"\x02d9"   L"Point en chef",
    L"\x0311"   L"Brve renverse",
    L"\xa704"   L"Trma et Accent aigu",
    L"\xa705"   L"Trma et Accent grave",
    L"\xa706"   L"Trma et Macron",
    L"\xa707"   L"Trma et Caron",
    L"\xa708"	L"Brve et Accent grave",
    L"\xa709"	L"Brve et Accent aigu",
    L"\xa70a"	L"Brve et Point souscrit",
    L"\xa70b"	L"Brve et Tilde",
    L"\xa70c"	L"Brve et Crochet en chef",
    L"\xa70d"	L"Circonflexe et Accent grave",
    L"\xa70e"	L"Circonflexe et Accent aigu",
    L"\xa70f"	L"Circonflexe et Point souscrit",
    L"\xa710"	L"Circonflexe et Tilde",
    L"\xa711"	L"Circonflexe et Crochet en chef",
    L"\xa712"	L"Cornu et Accent grave",
    L"\xa713"	L"Cornu et Accent aigu",
    L"\xa714"	L"Cornu et Point souscrit",
    L"\xa715"	L"Cornu et Tilde",
    L"\xa716"	L"Cornu et Crochet en chef",
    NULL
};

static ALLOC_SECTION_LDATA DEADKEY aDeadKey[] = {

/* Circonflexe */
    DEADTRANS( L'c'   , 0x02c6 , 0x0109 , 0x0000),
    DEADTRANS( L'a'   , 0x02c6 , 0x00e2 , 0x0000),
    DEADTRANS( L'h'   , 0x02c6 , 0x0125 , 0x0000),
    DEADTRANS( L'j'   , 0x02c6 , 0x0135 , 0x0000),
    DEADTRANS( L'g'   , 0x02c6 , 0x011d , 0x0000),
    DEADTRANS( L's'   , 0x02c6 , 0x015d , 0x0000),
    DEADTRANS( L'w'   , 0x02c6 , 0x0175 , 0x0000),
    DEADTRANS( L'e'   , 0x02c6 , 0x00ea , 0x0000),
    DEADTRANS( L'u'   , 0x02c6 , 0x00fb , 0x0000),
    DEADTRANS( L'i'   , 0x02c6 , 0x00ee , 0x0000),
    DEADTRANS( L'y'   , 0x02c6 , 0x0177 , 0x0000),
    DEADTRANS( L'o'   , 0x02c6 , 0x00f4 , 0x0000),
    DEADTRANS( L'C'   , 0x02c6 , 0x0108 , 0x0000),
    DEADTRANS( L'A'   , 0x02c6 , 0x00c2 , 0x0000),
    DEADTRANS( L'H'   , 0x02c6 , 0x0124 , 0x0000),
    DEADTRANS( L'J'   , 0x02c6 , 0x0134 , 0x0000),
    DEADTRANS( L'G'   , 0x02c6 , 0x011c , 0x0000),
    DEADTRANS( L'S'   , 0x02c6 , 0x015c , 0x0000),
    DEADTRANS( L'W'   , 0x02c6 , 0x0174 , 0x0000),
    DEADTRANS( L'E'   , 0x02c6 , 0x00ca , 0x0000),
    DEADTRANS( L'U'   , 0x02c6 , 0x00db , 0x0000),
    DEADTRANS( L'I'   , 0x02c6 , 0x00ce , 0x0000),
    DEADTRANS( L'Y'   , 0x02c6 , 0x0176 , 0x0000),
    DEADTRANS( L'O'   , 0x02c6 , 0x00d4 , 0x0000),
    DEADTRANS( L'Z'   , 0x02c6 , 0x1e90 , 0x0000),
    DEADTRANS( L'z'   , 0x02c6 , 0x1e91 , 0x0000),
    DEADTRANS( L' '   , 0x02c6 , 0x02c6 , 0x0000),
    DEADTRANS( L'`'   , 0x02c6 , 0xa70d , DKF_DEAD),  /* circonflexe et accent grave */
    DEADTRANS( 0x00b4 , 0x02c6 , 0xa70e , DKF_DEAD),  /* circonflexe et accent aigu */
    DEADTRANS( 0x0323 , 0x02c6 , 0xa70f , DKF_DEAD),  /* circonflexe et point souscrit */
    DEADTRANS( L'~'   , 0x02c6 , 0xa710 , DKF_DEAD),  /* circonflexe et tilde */
    DEADTRANS( 0x02c0 , 0x02c6 , 0xa711 , DKF_DEAD),  /* circonflexe et crochet en chef */

/* Trma */
    DEADTRANS( L'a'   , 0x00a8 , 0x00e4 , 0x0000),
    DEADTRANS( L'e'   , 0x00a8 , 0x00eb , 0x0000),
    DEADTRANS( L'u'   , 0x00a8 , 0x00fc , 0x0000),
    DEADTRANS( L'i'   , 0x00a8 , 0x00ef , 0x0000),
    DEADTRANS( L'y'   , 0x00a8 , 0x00ff , 0x0000),
    DEADTRANS( L'o'   , 0x00a8 , 0x00f6 , 0x0000),
    DEADTRANS( L'A'   , 0x00a8 , 0x00c4 , 0x0000),
    DEADTRANS( L'E'   , 0x00a8 , 0x00cb , 0x0000),
    DEADTRANS( L'U'   , 0x00a8 , 0x00dc , 0x0000),
    DEADTRANS( L'I'   , 0x00a8 , 0x00cf , 0x0000),
    DEADTRANS( L'Y'   , 0x00a8 , 0x0178 , 0x0000),
    DEADTRANS( L'O'   , 0x00a8 , 0x00d6 , 0x0000),
    DEADTRANS( L'H'   , 0x00a8 , 0x1e26 , 0x0000),
    DEADTRANS( L'h'   , 0x00a8 , 0x1e27 , 0x0000),
    DEADTRANS( L't'   , 0x00a8 , 0x1e97 , 0x0000),
    DEADTRANS( L'W'   , 0x00a8 , 0x1e84 , 0x0000),
    DEADTRANS( L'w'   , 0x00a8 , 0x1e85 , 0x0000),
    DEADTRANS( L'X'   , 0x00a8 , 0x1e8c , 0x0000),
    DEADTRANS( L'x'   , 0x00a8 , 0x1e8d , 0x0000),
    DEADTRANS( L' '   , 0x00a8 , 0x00a8 , 0x0000),
    DEADTRANS( L'`'   , 0x00a8 , 0xa705 , DKF_DEAD),  /* trma et accent grave */
    DEADTRANS( 0x00b4 , 0x00a8 , 0xa704 , DKF_DEAD),  /* trma et accent aigu */
    DEADTRANS( 0x02c7 , 0x00a8 , 0xa707 , DKF_DEAD),  /* trma et caron */
    DEADTRANS( 0x02c9 , 0x00a8 , 0xa706 , DKF_DEAD),  /* trma et macron */

/* Accent grave */
    DEADTRANS( L'a'   , L'`'   , 0x00e0 , 0x0000),
    DEADTRANS( L'e'   , L'`'   , 0x00e8 , 0x0000),
    DEADTRANS( L'u'   , L'`'   , 0x00f9 , 0x0000),
    DEADTRANS( L'i'   , L'`'   , 0x00ec , 0x0000),
    DEADTRANS( L'o'   , L'`'   , 0x00f2 , 0x0000),
    DEADTRANS( L'y'   , L'`'   , 0x1ef3 , 0x0000), 
    DEADTRANS( L'A'   , L'`'   , 0x00c0 , 0x0000),
    DEADTRANS( L'E'   , L'`'   , 0x00c8 , 0x0000),
    DEADTRANS( L'U'   , L'`'   , 0x00d9 , 0x0000),
    DEADTRANS( L'I'   , L'`'   , 0x00cc , 0x0000),
    DEADTRANS( L'O'   , L'`'   , 0x00d2 , 0x0000),
    DEADTRANS( L'Y'   , L'`'   , 0x1ef2 , 0x0000),
    DEADTRANS( L'N'   , L'`'   , 0x01f8 , 0x0000),
    DEADTRANS( L'n'   , L'`'   , 0x01f9 , 0x0000),
    DEADTRANS( L'W'   , L'`'   , 0x1e80 , 0x0000),
    DEADTRANS( L'w'   , L'`'   , 0x1e81 , 0x0000),
    DEADTRANS( L' '   , L'`'   , L'`'   , 0x0000),
    DEADTRANS( 0x00a8 , L'`'   , 0xa705 , DKF_DEAD),  /* accent grave et trma */
    DEADTRANS( 0x02d8 , L'`'   , 0xa708 , DKF_DEAD),  /* accent grave et brve */
    DEADTRANS( 0x02c6 , L'`'   , 0xa70d , DKF_DEAD),  /* accent grave et circonflexe */
    DEADTRANS( 0x0323 , L'`'   , 0xa712 , DKF_DEAD),  /* accent grave et cornu */

/* Tilde */
    DEADTRANS( L' '   , L'~'   , L'~'   , 0x0000),
    DEADTRANS( L'n'   , L'~'   , 0x00f1 , 0x0000),
    DEADTRANS( L'o'   , L'~'   , 0x00f5 , 0x0000),
    DEADTRANS( L'a'   , L'~'   , 0x00e3 , 0x0000),
    DEADTRANS( L'e'   , L'~'   , 0x1ebd , 0x0000),
    DEADTRANS( L'i'   , L'~'   , 0x0129 , 0x0000),
    DEADTRANS( L'u'   , L'~'   , 0x0169 , 0x0000),
    DEADTRANS( L'y'   , L'~'   , 0x1ef9 , 0x0000),
    DEADTRANS( L'N'   , L'~'   , 0x00d1 , 0x0000),
    DEADTRANS( L'O'   , L'~'   , 0x00d5 , 0x0000),
    DEADTRANS( L'A'   , L'~'   , 0x00c3 , 0x0000),
    DEADTRANS( L'E'   , L'~'   , 0x1ebc , 0x0000),   
    DEADTRANS( L'I'   , L'~'   , 0x0128 , 0x0000),
    DEADTRANS( L'U'   , L'~'   , 0x0168 , 0x0000),
    DEADTRANS( L'Y'   , L'~'   , 0x1ef8 , 0x0000),
    DEADTRANS( L'V'   , L'~'   , 0x1e7c , 0x0000),
    DEADTRANS( L'v'   , L'~'   , 0x1e7d , 0x0000),
    DEADTRANS( 0x02d8 , L'~'   , 0xA70b , DKF_DEAD),  /* tilde et brve */
    DEADTRANS( 0x02c6 , L'~'   , 0xa710 , DKF_DEAD),  /* tilde et circonflexe */
    DEADTRANS( 0x02bc , L'~'   , 0xa715 , DKF_DEAD),  /* tilde et cornu */

/* Accent aigu */
    DEADTRANS( L' '   , 0x00b4 , 0x00b4 , 0x0000),
    DEADTRANS( L'e'   , 0x00b4 , 0x00e9 , 0x0000),
    DEADTRANS( L'u'   , 0x00b4 , 0x00fa , 0x0000),
    DEADTRANS( L'i'   , 0x00b4 , 0x00ed , 0x0000),
    DEADTRANS( L'y'   , 0x00b4 , 0x00fd , 0x0000),
    DEADTRANS( L'o'   , 0x00b4 , 0x00f3 , 0x0000),
    DEADTRANS( L'a'   , 0x00b4 , 0x00e1 , 0x0000),
    DEADTRANS( L'E'   , 0x00b4 , 0x00c9 , 0x0000),
    DEADTRANS( L'U'   , 0x00b4 , 0x00da , 0x0000),
    DEADTRANS( L'I'   , 0x00b4 , 0x00cd , 0x0000),
    DEADTRANS( L'Y'   , 0x00b4 , 0x00dd , 0x0000),
    DEADTRANS( L'O'   , 0x00b4 , 0x00d3 , 0x0000),
    DEADTRANS( L'A'   , 0x00b4 , 0x00c1 , 0x0000),
    DEADTRANS( L'C'   , 0x00b4 , 0x0106 , 0x0000),
    DEADTRANS( L'c'   , 0x00b4 , 0x0107 , 0x0000),
    DEADTRANS( L'G'   , 0x00b4 , 0x01f4 , 0x0000),
    DEADTRANS( L'g'   , 0x00b4 , 0x01f5 , 0x0000),
    DEADTRANS( L'K'   , 0x00b4 , 0x1e30 , 0x0000),
    DEADTRANS( L'k'   , 0x00b4 , 0x1e31 , 0x0000),
    DEADTRANS( L'L'   , 0x00b4 , 0x0139 , 0x0000),
    DEADTRANS( L'l'   , 0x00b4 , 0x013a , 0x0000),
    DEADTRANS( L'M'   , 0x00b4 , 0x1e3e , 0x0000),
    DEADTRANS( L'm'   , 0x00b4 , 0x1e3f , 0x0000),
    DEADTRANS( L'N'   , 0x00b4 , 0x0143 , 0x0000),
    DEADTRANS( L'n'   , 0x00b4 , 0x0144 , 0x0000),
    DEADTRANS( L'P'   , 0x00b4 , 0x1e54 , 0x0000),
    DEADTRANS( L'p'   , 0x00b4 , 0x1e55 , 0x0000),
    DEADTRANS( L'R'   , 0x00b4 , 0x0154 , 0x0000),
    DEADTRANS( L'r'   , 0x00b4 , 0x0155 , 0x0000),
    DEADTRANS( L'S'   , 0x00b4 , 0x015a , 0x0000),
    DEADTRANS( L's'   , 0x00b4 , 0x015b , 0x0000),
    DEADTRANS( L'W'   , 0x00b4 , 0x1e82 , 0x0000),
    DEADTRANS( L'w'   , 0x00b4 , 0x1e83 , 0x0000),
    DEADTRANS( L'Z'   , 0x00b4 , 0x0179 , 0x0000),
    DEADTRANS( L'z'   , 0x00b4 , 0x017a , 0x0000),
    DEADTRANS( 0x00a8 , 0x00b4 , 0xa704 , DKF_DEAD),  /* accent aigu et trma */
    DEADTRANS( 0x02d8 , 0x00b4 , 0xa709 , DKF_DEAD),  /* accent aigu et brve */
    DEADTRANS( 0x02c6 , 0x00b4 , 0xa70e , DKF_DEAD),  /* accent aigu et circonflexe */
    DEADTRANS( 0x02bc , 0x00b4 , 0xa713 , DKF_DEAD),  /* accent aigu et cornu */ 

/* Caron */
    DEADTRANS( L' '   , 0x02c7 , 0x02c7 , 0x0000),
    DEADTRANS( L'a'   , 0x02c7 , 0x01ce , 0x0000),
    DEADTRANS( L'e'   , 0x02c7 , 0x011b , 0x0000),
    DEADTRANS( L'o'   , 0x02c7 , 0x01d2 , 0x0000),
    DEADTRANS( L'u'   , 0x02c7 , 0x01d4 , 0x0000),
    DEADTRANS( L'A'   , 0x02c7 , 0x01cd , 0x0000),
    DEADTRANS( L'E'   , 0x02c7 , 0x011a , 0x0000),
    DEADTRANS( L'O'   , 0x02c7 , 0x01d1 , 0x0000),
    DEADTRANS( L'U'   , 0x02c7 , 0x01d3 , 0x0000),
    DEADTRANS( L'C'   , 0x02c7 , 0x010c , 0x0000),
    DEADTRANS( L'c'   , 0x02c7 , 0x010d , 0x0000),
    DEADTRANS( L'D'   , 0x02c7 , 0x010e , 0x0000),
    DEADTRANS( L'd'   , 0x02c7 , 0x010f , 0x0000),
    DEADTRANS( L'G'   , 0x02c7 , 0x01e6 , 0x0000),
    DEADTRANS( L'g'   , 0x02c7 , 0x01e7 , 0x0000),
    DEADTRANS( L'H'   , 0x02c7 , 0x021e , 0x0000),
    DEADTRANS( L'h'   , 0x02c7 , 0x021f , 0x0000),
    DEADTRANS( L'I'   , 0x02c7 , 0x01cf , 0x0000),
    DEADTRANS( L'i'   , 0x02c7 , 0x01d0 , 0x0000),
    DEADTRANS( L'j'   , 0x02c7 , 0x01f0 , 0x0000),
    DEADTRANS( L'K'   , 0x02c7 , 0x01e8 , 0x0000),
    DEADTRANS( L'k'   , 0x02c7 , 0x01e9 , 0x0000),
    DEADTRANS( L'L'   , 0x02c7 , 0x013d , 0x0000),
    DEADTRANS( L'l'   , 0x02c7 , 0x013e , 0x0000),
    DEADTRANS( L'N'   , 0x02c7 , 0x0147 , 0x0000),
    DEADTRANS( L'n'   , 0x02c7 , 0x0148 , 0x0000),
    DEADTRANS( L'R'   , 0x02c7 , 0x0158 , 0x0000),
    DEADTRANS( L'r'   , 0x02c7 , 0x0159 , 0x0000),
    DEADTRANS( L'S'   , 0x02c7 , 0x0160 , 0x0000),
    DEADTRANS( L's'   , 0x02c7 , 0x0161 , 0x0000),
    DEADTRANS( L'T'   , 0x02c7 , 0x0164 , 0x0000),
    DEADTRANS( L't'   , 0x02c7 , 0x0165 , 0x0000),
    DEADTRANS( L'Z'   , 0x02c7 , 0x017d , 0x0000),
    DEADTRANS( L'z'   , 0x02c7 , 0x017e , 0x0000),
    DEADTRANS( 0x00a8 , 0x02c7 , 0xa707 , DKF_DEAD),  /* caron et trma */
   
/* Macron */
    DEADTRANS( L' '   , 0x02c9 , 0x02c9 , 0x0000),
    DEADTRANS( L'a'   , 0x02c9 , 0x0101 , 0x0000),
    DEADTRANS( L'e'   , 0x02c9 , 0x0113 , 0x0000),
    DEADTRANS( L'i'   , 0x02c9 , 0x012b , 0x0000),
    DEADTRANS( L'o'   , 0x02c9 , 0x014d , 0x0000),
    DEADTRANS( L'u'   , 0x02c9 , 0x016b , 0x0000),
    DEADTRANS( L'A'   , 0x02c9 , 0x0100 , 0x0000),
    DEADTRANS( L'E'   , 0x02c9 , 0x0112 , 0x0000),
    DEADTRANS( L'I'   , 0x02c9 , 0x012a , 0x0000),
    DEADTRANS( L'O'   , 0x02c9 , 0x014c , 0x0000),
    DEADTRANS( L'U'   , 0x02c9 , 0x016a , 0x0000),
    DEADTRANS( L'G'   , 0x02c9 , 0x1e20 , 0x0000),
    DEADTRANS( L'g'   , 0x02c9 , 0x1e21 , 0x0000),
    DEADTRANS( L'Y'   , 0x02c9 , 0x0232 , 0x0000),
    DEADTRANS( L'y'   , 0x02c9 , 0x0233 , 0x0000),
    DEADTRANS( L''   , 0x02c9 , 0x01e2 , 0x0000),
    DEADTRANS( L''   , 0x02c9 , 0x01e3 , 0x0000),

    DEADTRANS( 0x00a8 , 0x02c9 , 0xa706 , DKF_DEAD),  /* macron et trma */ 

/* Brve */
    DEADTRANS( L' '   , 0x02d8 , 0x02d8 , 0x0000),
    DEADTRANS( L'a'   , 0x02d8 , 0x0103 , 0x0000),
    DEADTRANS( L'A'   , 0x02d8 , 0x0102 , 0x0000),
    DEADTRANS( L'E'   , 0x02d8 , 0x0114 , 0x0000),
    DEADTRANS( L'e'   , 0x02d8 , 0x0115 , 0x0000),
    DEADTRANS( L'G'   , 0x02d8 , 0x011e , 0x0000),
    DEADTRANS( L'g'   , 0x02d8 , 0x011f , 0x0000),
    DEADTRANS( L'I'   , 0x02d8 , 0x012c , 0x0000),
    DEADTRANS( L'i'   , 0x02d8 , 0x012d , 0x0000),
    DEADTRANS( L'O'   , 0x02d8 , 0x014e , 0x0000),
    DEADTRANS( L'o'   , 0x02d8 , 0x014f , 0x0000),
    DEADTRANS( L'U'   , 0x02d8 , 0x016c , 0x0000),
    DEADTRANS( L'u'   , 0x02d8 , 0x016d , 0x0000),
    DEADTRANS( L'`'   , 0x02d8 , 0xa708 , DKF_DEAD),  /* brve et accent grave */
    DEADTRANS( 0x00b4 , 0x02d8 , 0xa709 , DKF_DEAD),  /* brve et accent aigu */
    DEADTRANS( 0x0323 , 0x02d8 , 0xa70a , DKF_DEAD),  /* brve et point souscrit */
    DEADTRANS( L'~'   , 0x02d8 , 0xa70b , DKF_DEAD),  /* brve et tilde */
    DEADTRANS( 0x02c0 , 0x02d8 , 0xa70c , DKF_DEAD),  /* brve et crochet en chef */

/* Point souscrit */
    DEADTRANS( L' '   , 0x0323 , 0x0323 , 0x0000),
    DEADTRANS( L'a'   , 0x0323 , 0x1ea1 , 0x0000),
    DEADTRANS( L'A'   , 0x0323 , 0x1ea0 , 0x0000),
    DEADTRANS( L'e'   , 0x0323 , 0x1eb9 , 0x0000),
    DEADTRANS( L'E'   , 0x0323 , 0x1eb8 , 0x0000),
    DEADTRANS( L'i'   , 0x0323 , 0x1ecb , 0x0000),
    DEADTRANS( L'I'   , 0x0323 , 0x1eca , 0x0000),
    DEADTRANS( L'o'   , 0x0323 , 0x1ecd , 0x0000),
    DEADTRANS( L'O'   , 0x0323 , 0x1ecc , 0x0000),
    DEADTRANS( L'u'   , 0x0323 , 0x1ee5 , 0x0000),
    DEADTRANS( L'U'   , 0x0323 , 0x1ee4 , 0x0000),
    DEADTRANS( L'y'   , 0x0323 , 0x1ef5 , 0x0000),
    DEADTRANS( L'Y'   , 0x0323 , 0x1ef4 , 0x0000),
    DEADTRANS( L'B'   , 0x0323 , 0x1e04 , 0x0000),
    DEADTRANS( L'b'   , 0x0323 , 0x1e05 , 0x0000),
    DEADTRANS( L'n'   , 0x0323 , 0x1e47 , 0x0000),
    DEADTRANS( L'v'   , 0x0323 , 0x1e7f , 0x0000),
    DEADTRANS( L'w'   , 0x0323 , 0x1e89 , 0x0000),
    DEADTRANS( L'h'   , 0x0323 , 0x1e25 , 0x0000),
    DEADTRANS( L'd'   , 0x0323 , 0x1e0d , 0x0000),
    DEADTRANS( L'm'   , 0x0323 , 0x1e43 , 0x0000),
    DEADTRANS( L's'   , 0x0323 , 0x1e63 , 0x0000),
    DEADTRANS( L'l'   , 0x0323 , 0x1e37 , 0x0000),
    DEADTRANS( L'k'   , 0x0323 , 0x1e33 , 0x0000),
    DEADTRANS( L'z'   , 0x0323 , 0x1e93 , 0x0000),
    DEADTRANS( L'r'   , 0x0323 , 0x1e5b , 0x0000),
    DEADTRANS( L't'   , 0x0323 , 0x1e6d , 0x0000),
    DEADTRANS( L'N'   , 0x0323 , 0x1e46 , 0x0000),
    DEADTRANS( L'V'   , 0x0323 , 0x1e7e , 0x0000),
    DEADTRANS( L'W'   , 0x0323 , 0x1e88 , 0x0000),
    DEADTRANS( L'H'   , 0x0323 , 0x1e24 , 0x0000),
    DEADTRANS( L'D'   , 0x0323 , 0x1e0c , 0x0000),
    DEADTRANS( L'M'   , 0x0323 , 0x1e42 , 0x0000),
    DEADTRANS( L'S'   , 0x0323 , 0x1e62 , 0x0000),
    DEADTRANS( L'L'   , 0x0323 , 0x1e36 , 0x0000),
    DEADTRANS( L'K'   , 0x0323 , 0x1e32 , 0x0000),
    DEADTRANS( L'Z'   , 0x0323 , 0x1e92 , 0x0000),
    DEADTRANS( L'R'   , 0x0323 , 0x1e5a , 0x0000),
    DEADTRANS( L'T'   , 0x0323 , 0x1e6c , 0x0000),
    DEADTRANS( L'Y'   , 0x0323 , 0x1ef4 , 0x0000),
    DEADTRANS( 0x02d8 , 0x0323 , 0xa70a , DKF_DEAD),  /* point souscrit et brve */
    DEADTRANS( 0x02c6 , 0x0323 , 0xa70f , DKF_DEAD),  /* point souscrit et circonflexe */
    DEADTRANS( 0x02bc , 0x0323 , 0xa714 , DKF_DEAD),  /* point souscrit et cornu */

/* Cornu */
    DEADTRANS( L' '   , 0x02bc , 0x02bc , 0x0000),
    DEADTRANS( L'o'   , 0x02bc , 0x01a1 , 0x0000),
    DEADTRANS( L'O'   , 0x02bc , 0x01a0 , 0x0000),
    DEADTRANS( L'u'   , 0x02bc , 0x01b0 , 0x0000),
    DEADTRANS( L'U'   , 0x02bc , 0x01af , 0x0000),
    DEADTRANS( L'`'   , 0x02bc , 0xa712 , DKF_DEAD),  /* cornu et accent grave */
    DEADTRANS( 0x00b4 , 0x02bc , 0xa713 , DKF_DEAD),  /* cornu et accent aigu */
    DEADTRANS( 0x0323 , 0x02bc , 0xa714 , DKF_DEAD),  /* cornu et point souscrit */
    DEADTRANS( L'~'   , 0x02bc , 0xa715 , DKF_DEAD),  /* cornu et tilde */
    DEADTRANS( 0x02c0 , 0x02bc , 0xa716 , DKF_DEAD),  /* cornu et crochet en chef */

/* Crochet en chef */
    DEADTRANS( L' '   , 0x02c0 , 0x02c0 , 0x0000),
    DEADTRANS( L'a'   , 0x02c0 , 0x1ea3 , 0x0000),
    DEADTRANS( L'A'   , 0x02c0 , 0x1ea2 , 0x0000),
    DEADTRANS( L'e'   , 0x02c0 , 0x1ebb , 0x0000),
    DEADTRANS( L'E'   , 0x02c0 , 0x1eba , 0x0000),
    DEADTRANS( L'i'   , 0x02c0 , 0x1ec9 , 0x0000),
    DEADTRANS( L'I'   , 0x02c0 , 0x1ec8 , 0x0000),
    DEADTRANS( L'o'   , 0x02c0 , 0x1ecf , 0x0000),
    DEADTRANS( L'O'   , 0x02c0 , 0x1ece , 0x0000),
    DEADTRANS( L'u'   , 0x02c0 , 0x1ee7 , 0x0000),
    DEADTRANS( L'U'   , 0x02c0 , 0x1ee6 , 0x0000),
    DEADTRANS( L'y'   , 0x02c0 , 0x1ef7 , 0x0000),
    DEADTRANS( L'Y'   , 0x02c0 , 0x1ef6 , 0x0000),
    DEADTRANS( 0x02d8 , 0x02c0 , 0xa70c , DKF_DEAD),   /* crochet et brve */
    DEADTRANS( 0x02c6 , 0x02c0 , 0xa711 , DKF_DEAD),   /* crochet et circonflexe */
    DEADTRANS( 0x02bc , 0x02c0 , 0xa716 , DKF_DEAD),   /* crochet et cornu */

/* Rond en chef */
    DEADTRANS( L'u'   , 0x02da , 0x016f , 0x0000),
    DEADTRANS( L'y'   , 0x02da , 0x1e99 , 0x0000),
    DEADTRANS( L'a'   , 0x02da , 0x00e5 , 0x0000),
    DEADTRANS( L'w'   , 0x02da , 0x1e98 , 0x0000),
    DEADTRANS( L'U'   , 0x02da , 0x016e , 0x0000),
    DEADTRANS( L'A'   , 0x02da , 0x00c5 , 0x0000),
    DEADTRANS( L' '   , 0x02da , 0x02da , 0x0000),

/* Barre */
    DEADTRANS( L'h'   , 0x0336 , 0x0127 , 0x0000),
    DEADTRANS( L'g'   , 0x0336 , 0x01e5 , 0x0000),
    DEADTRANS( L'd'   , 0x0336 , 0x0111 , 0x0000),
    DEADTRANS( L'l'   , 0x0336 , 0x0142 , 0x0000),
    DEADTRANS( L'z'   , 0x0336 , 0x01b6 , 0x0000),
    DEADTRANS( L't'   , 0x0336 , 0x0167 , 0x0000),
    DEADTRANS( L'o'   , 0x0336 , 0x00f8 , 0x0000),
    DEADTRANS( L'H'   , 0x0336 , 0x0126 , 0x0000),
    DEADTRANS( L'G'   , 0x0336 , 0x01e4 , 0x0000),
    DEADTRANS( L'D'   , 0x0336 , 0x0110 , 0x0000),
    DEADTRANS( L'L'   , 0x0336 , 0x0141 , 0x0000),
    DEADTRANS( L'Z'   , 0x0336 , 0x01b5 , 0x0000),
    DEADTRANS( L'T'   , 0x0336 , 0x0166 , 0x0000),
    DEADTRANS( L'O'   , 0x0336 , 0x00d8 , 0x0000),
    DEADTRANS( L' '   , 0x0336 , 0x0336 , 0x0000),

/* Double accent aigu */
    DEADTRANS( L'u'   , 0x02dd , 0x0171 , 0x0000),
    DEADTRANS( L'o'   , 0x02dd , 0x0151 , 0x0000),
    DEADTRANS( L'U'   , 0x02dd , 0x0170 , 0x0000),
    DEADTRANS( L'O'   , 0x02dd , 0x0150 , 0x0000),
    DEADTRANS( L' '   , 0x02dd , 0x02dd , 0x0000),

/* Double accent grave */
    DEADTRANS( L'e'   , 0x030f , 0x0205 , 0x0000),
    DEADTRANS( L'r'   , 0x030f , 0x0211 , 0x0000),
    DEADTRANS( L'u'   , 0x030f , 0x0215 , 0x0000),
    DEADTRANS( L'i'   , 0x030f , 0x0209 , 0x0000),
    DEADTRANS( L'o'   , 0x030f , 0x020d , 0x0000),
    DEADTRANS( L'a'   , 0x030f , 0x0201 , 0x0000),
    DEADTRANS( L'E'   , 0x030f , 0x0204 , 0x0000),
    DEADTRANS( L'R'   , 0x030f , 0x0210 , 0x0000),
    DEADTRANS( L'U'   , 0x030f , 0x0214 , 0x0000),
    DEADTRANS( L'I'   , 0x030f , 0x0208 , 0x0000),
    DEADTRANS( L'O'   , 0x030f , 0x020c , 0x0000),
    DEADTRANS( L'A'   , 0x030f , 0x0200 , 0x0000),
    DEADTRANS( L' '   , 0x030f , 0x030f , 0x0000),

/* Ogonek */
    DEADTRANS( L'e'   , 0x02db , 0x0119 , 0x0000),
    DEADTRANS( L'u'   , 0x02db , 0x0173 , 0x0000),
    DEADTRANS( L'i'   , 0x02db , 0x012f , 0x0000),
    DEADTRANS( L'o'   , 0x02db , 0x01eb , 0x0000),
    DEADTRANS( L'a'   , 0x02db , 0x0105 , 0x0000),
    DEADTRANS( L'E'   , 0x02db , 0x0118 , 0x0000),
    DEADTRANS( L'U'   , 0x02db , 0x0172 , 0x0000),
    DEADTRANS( L'I'   , 0x02db , 0x012e , 0x0000),
    DEADTRANS( L'O'   , 0x02db , 0x01ea , 0x0000),
    DEADTRANS( L'A'   , 0x02db , 0x0104 , 0x0000),
    DEADTRANS( L' '   , 0x02db , 0x0328 , 0x0000),

/* Macron souscrit */
    DEADTRANS( L'b'   , 0x02cd , 0x1e07 , 0x0000),
    DEADTRANS( L'n'   , 0x02cd , 0x1e49 , 0x0000),
    DEADTRANS( L'h'   , 0x02cd , 0x1e96 , 0x0000),
    DEADTRANS( L'd'   , 0x02cd , 0x1e0f , 0x0000),
    DEADTRANS( L'l'   , 0x02cd , 0x1e3b , 0x0000),
    DEADTRANS( L'k'   , 0x02cd , 0x1e35 , 0x0000),
    DEADTRANS( L'z'   , 0x02cd , 0x1e95 , 0x0000),
    DEADTRANS( L'r'   , 0x02cd , 0x1e5f , 0x0000),
    DEADTRANS( L't'   , 0x02cd , 0x1e6f , 0x0000),
    DEADTRANS( L'B'   , 0x02cd , 0x1e06 , 0x0000),
    DEADTRANS( L'N'   , 0x02cd , 0x1e48 , 0x0000),
    DEADTRANS( L'D'   , 0x02cd , 0x1e0e , 0x0000),
    DEADTRANS( L'L'   , 0x02cd , 0x1e3a , 0x0000),
    DEADTRANS( L'K'   , 0x02cd , 0x1e34 , 0x0000),
    DEADTRANS( L'Z'   , 0x02cd , 0x1e94 , 0x0000),
    DEADTRANS( L'R'   , 0x02cd , 0x1e5e , 0x0000),
    DEADTRANS( L'T'   , 0x02cd , 0x1e6e , 0x0000),
    DEADTRANS( L' '   , 0x02cd , 0x02cd , 0x0000),

/* Cdille */
    DEADTRANS( L'n'   , 0x00b8 , 0x0146 , 0x0000),
    DEADTRANS( L'c'   , 0x00b8 , 0x00e7 , 0x0000),
    DEADTRANS( L'h'   , 0x00b8 , 0x1e29 , 0x0000),
    DEADTRANS( L'g'   , 0x00b8 , 0x0123 , 0x0000),
    DEADTRANS( L'd'   , 0x00b8 , 0x1e11 , 0x0000),
    DEADTRANS( L's'   , 0x00b8 , 0x015f , 0x0000),
    DEADTRANS( L'l'   , 0x00b8 , 0x013c , 0x0000),
    DEADTRANS( L'k'   , 0x00b8 , 0x0136 , 0x0000),
    DEADTRANS( L'e'   , 0x00b8 , 0x0229 , 0x0000),
    DEADTRANS( L'r'   , 0x00b8 , 0x0157 , 0x0000),
    DEADTRANS( L't'   , 0x00b8 , 0x0163 , 0x0000),
    DEADTRANS( L'N'   , 0x00b8 , 0x0145 , 0x0000),
    DEADTRANS( L'C'   , 0x00b8 , 0x00c7 , 0x0000),
    DEADTRANS( L'H'   , 0x00b8 , 0x1e28 , 0x0000),
    DEADTRANS( L'G'   , 0x00b8 , 0x0122 , 0x0000),
    DEADTRANS( L'D'   , 0x00b8 , 0x1e10 , 0x0000),
    DEADTRANS( L'S'   , 0x00b8 , 0x015e , 0x0000),
    DEADTRANS( L'L'   , 0x00b8 , 0x013b , 0x0000),
    DEADTRANS( L'K'   , 0x00b8 , 0x0137 , 0x0000),
    DEADTRANS( L'E'   , 0x00b8 , 0x0228 , 0x0000),
    DEADTRANS( L'R'   , 0x00b8 , 0x0156 , 0x0000),
    DEADTRANS( L'T'   , 0x00b8 , 0x0162 , 0x0000),
    DEADTRANS( L' '   , 0x00b8 , 0x00b8 , 0x0000),

/* Point en chef */
    DEADTRANS( L'b'   , 0x02d9 , 0x1e03 , 0x0000),
    DEADTRANS( L'n'   , 0x02d9 , 0x1e45 , 0x0000),
    DEADTRANS( L'c'   , 0x02d9 , 0x010b , 0x0000),
    DEADTRANS( L'x'   , 0x02d9 , 0x1e8b , 0x0000),
    DEADTRANS( L'w'   , 0x02d9 , 0x1e87 , 0x0000),
    DEADTRANS( L'h'   , 0x02d9 , 0x1e23 , 0x0000),
    DEADTRANS( L'g'   , 0x02d9 , 0x0121 , 0x0000),
    DEADTRANS( L'f'   , 0x02d9 , 0x1e1f , 0x0000),
    DEADTRANS( L'd'   , 0x02d9 , 0x1e0b , 0x0000),
    DEADTRANS( L'm'   , 0x02d9 , 0x1e41 , 0x0000),
    DEADTRANS( L's'   , 0x02d9 , 0x1e61 , 0x0000),
    DEADTRANS( L'z'   , 0x02d9 , 0x017c , 0x0000),
    DEADTRANS( L'e'   , 0x02d9 , 0x0117 , 0x0000),
    DEADTRANS( L'r'   , 0x02d9 , 0x1e59 , 0x0000),
    DEADTRANS( L'p'   , 0x02d9 , 0x1e57 , 0x0000),
    DEADTRANS( L'i'   , 0x02d9 , 0x0131 , 0x0000),
    DEADTRANS( L't'   , 0x02d9 , 0x1e6b , 0x0000),
    DEADTRANS( L'y'   , 0x02d9 , 0x1e8f , 0x0000),
    DEADTRANS( L'o'   , 0x02d9 , 0x022f , 0x0000),
    DEADTRANS( L'a'   , 0x02d9 , 0x0227 , 0x0000),
    DEADTRANS( L'B'   , 0x02d9 , 0x1e02 , 0x0000),
    DEADTRANS( L'N'   , 0x02d9 , 0x1e44 , 0x0000),
    DEADTRANS( L'C'   , 0x02d9 , 0x010a , 0x0000),
    DEADTRANS( L'X'   , 0x02d9 , 0x1e8a , 0x0000),
    DEADTRANS( L'W'   , 0x02d9 , 0x1e86 , 0x0000),
    DEADTRANS( L'H'   , 0x02d9 , 0x1e22 , 0x0000),
    DEADTRANS( L'G'   , 0x02d9 , 0x0120 , 0x0000),
    DEADTRANS( L'F'   , 0x02d9 , 0x1e1e , 0x0000),
    DEADTRANS( L'D'   , 0x02d9 , 0x1e0a , 0x0000),
    DEADTRANS( L'M'   , 0x02d9 , 0x1e40 , 0x0000),
    DEADTRANS( L'S'   , 0x02d9 , 0x1e60 , 0x0000),
    DEADTRANS( L'Z'   , 0x02d9 , 0x017b , 0x0000),
    DEADTRANS( L'E'   , 0x02d9 , 0x0116 , 0x0000),
    DEADTRANS( L'R'   , 0x02d9 , 0x1e58 , 0x0000),
    DEADTRANS( L'P'   , 0x02d9 , 0x1e56 , 0x0000),
    DEADTRANS( L'I'   , 0x02d9 , 0x0130 , 0x0000),
    DEADTRANS( L'T'   , 0x02d9 , 0x1e6a , 0x0000),
    DEADTRANS( L'Y'   , 0x02d9 , 0x1e8e , 0x0000),
    DEADTRANS( L'O'   , 0x02d9 , 0x022e , 0x0000),
    DEADTRANS( L'A'   , 0x02d9 , 0x0226 , 0x0000),
    DEADTRANS( 0x017f , 0x02d9 , 0x1e9b , 0x0000),
    DEADTRANS( L' '   , 0x02d9 , 0x02d9 , 0x0000),

/* Brve renverse */
    DEADTRANS( L'e'   , 0x0311 , 0x0207 , 0x0000),
    DEADTRANS( L'r'   , 0x0311 , 0x0213 , 0x0000),
    DEADTRANS( L'u'   , 0x0311 , 0x0217 , 0x0000),
    DEADTRANS( L'i'   , 0x0311 , 0x020b , 0x0000),
    DEADTRANS( L'o'   , 0x0311 , 0x020f , 0x0000),
    DEADTRANS( L'a'   , 0x0311 , 0x0203 , 0x0000),
    DEADTRANS( L'E'   , 0x0311 , 0x0206 , 0x0000),
    DEADTRANS( L'R'   , 0x0311 , 0x0212 , 0x0000),
    DEADTRANS( L'U'   , 0x0311 , 0x0216 , 0x0000),
    DEADTRANS( L'I'   , 0x0311 , 0x020a , 0x0000),
    DEADTRANS( L'O'   , 0x0311 , 0x020e , 0x0000),
    DEADTRANS( L'A'   , 0x0311 , 0x0202 , 0x0000),
    DEADTRANS( L' '   , 0x0311 , 0x0311 , 0x0000),

 /* Trma et Accent aigu */
    DEADTRANS( L' '   , 0xa704 , 0xa704 , 0x0000), 
    DEADTRANS( L'u'   , 0xa704 , 0x01d8 , 0x0000),
    DEADTRANS( L'U'   , 0xa704 , 0x01d7 , 0x0000),

/* Trma et Accent grave */
    DEADTRANS( L' '   , 0xa705 , 0xa705 , 0x0000), 
    DEADTRANS( L'u'   , 0xa705 , 0x01dc , 0x0000),
    DEADTRANS( L'U'   , 0xa705 , 0x01db , 0x0000),

/* Trma et Macron */
    DEADTRANS( L' '   , 0xa706 , 0xa706 , 0x0000), 
    DEADTRANS( L'u'   , 0xa706 , 0x01d6 , 0x0000),
    DEADTRANS( L'U'   , 0xa706 , 0x01d5 , 0x0000),

/* Trma et Caron */
    DEADTRANS( L' '   , 0xa707 , 0xa707 , 0x0000), 
    DEADTRANS( L'u'   , 0xa707 , 0x01da , 0x0000),
    DEADTRANS( L'U'   , 0xa707 , 0x01d9 , 0x0000),

/* Brve et Accent grave */
DEADTRANS( L' '   , 0xa708 , 0xa708 , 0x0000),
DEADTRANS( L'a'   , 0xa708 , 0x1eb1 , 0x0000),
DEADTRANS( L'A'   , 0xa708 , 0x1eb0 , 0x0000),

/* Brve et Accent aigu */
DEADTRANS( L' '   , 0xa709 , 0xa709 , 0x0000),
DEADTRANS( L'a'   , 0xa709 , 0x1eaf , 0x0000),
DEADTRANS( L'A'   , 0xa709 , 0x1eae , 0x0000),

/* Brve et Point souscrit */
DEADTRANS( L' '   , 0xa70a , 0xa70a , 0x0000),
DEADTRANS( L'a'   , 0xa70a , 0x1eb7 , 0x0000),
DEADTRANS( L'A'   , 0xa70a , 0x1eb6 , 0x0000),

/* Brve et Tilde */
DEADTRANS( L' '   , 0xa70b , 0xa70b , 0x0000),
DEADTRANS( L'a'   , 0xa70b , 0x1eb5 , 0x0000),
DEADTRANS( L'A'   , 0xa70b , 0x1eb4 , 0x0000),

/* Brve et Crochet en chef */
DEADTRANS( L' '   , 0xa70c , 0xa70c , 0x0000),
DEADTRANS( L'a'   , 0xa70c , 0x1eb3 , 0x0000),
DEADTRANS( L'A'   , 0xa70c , 0x1eb2 , 0x0000), 

/* Circonflexe et Accent grave */
DEADTRANS( L' '   , 0xa70d , 0xa70d , 0x0000),
DEADTRANS( L'a'   , 0xa70d , 0x1ea7 , 0x0000),
DEADTRANS( L'A'   , 0xa70d , 0x1ea6 , 0x0000),
DEADTRANS( L'e'   , 0xa70d , 0x1ec1 , 0x0000),
DEADTRANS( L'E'   , 0xa70d , 0x1ec0 , 0x0000),
DEADTRANS( L'o'   , 0xa70d , 0x1ed3 , 0x0000),
DEADTRANS( L'O'   , 0xa70d , 0x1ed2 , 0x0000),

/* Circonflexe et Accent aigu */
 DEADTRANS( L' '   , 0xa70e , 0xa70e , 0x0000),
 DEADTRANS( L'a'   , 0xa70e , 0x1ea5 , 0x0000),
 DEADTRANS( L'A'   , 0xa70e , 0x1ea4 , 0x0000), 
 DEADTRANS( L'e'   , 0xa70e , 0x1ebf , 0x0000),
 DEADTRANS( L'E'   , 0xa70e , 0x1ebe , 0x0000),
 DEADTRANS( L'o'   , 0xa70e , 0x1ed1 , 0x0000),
 DEADTRANS( L'O'   , 0xa70e , 0x1ed0 , 0x0000),

/* Circonflexe et Point souscrit */
DEADTRANS( L' '   , 0xa70f , 0xa70f , 0x0000),
DEADTRANS( L'a'   , 0xa70f , 0x1ead , 0x0000),
DEADTRANS( L'A'   , 0xa70f , 0x1eac , 0x0000),
DEADTRANS( L'e'   , 0xa70f , 0x1ec7 , 0x0000),
DEADTRANS( L'E'   , 0xa70f , 0x1ec6 , 0x0000),
DEADTRANS( L'o'   , 0xa70f , 0x1ed9 , 0x0000),
DEADTRANS( L'O'   , 0xa70f , 0x1ed8 , 0x0000),

/* Circonflexe et Tilde */
DEADTRANS( L' '   , 0xa710 , 0xa710 , 0x0000),
DEADTRANS( L'a'   , 0xa710 , 0x1eab , 0x0000),
DEADTRANS( L'A'   , 0xa710 , 0x1eaa , 0x0000),
DEADTRANS( L'e'   , 0xa710 , 0x1ec5 , 0x0000),
DEADTRANS( L'E'   , 0xa710 , 0x1ec4 , 0x0000),
DEADTRANS( L'o'   , 0xa710 , 0x1ed7 , 0x0000),
DEADTRANS( L'O'   , 0xa710 , 0x1ed6 , 0x0000),

/* Circonflexe et Crochet en chef */
DEADTRANS( L' '   , 0xa711 , 0xa711 , 0x0000),
DEADTRANS( L'a'   , 0xa711 , 0x1ea9 , 0x0000),
DEADTRANS( L'A'   , 0xa711 , 0x1ea8 , 0x0000),
DEADTRANS( L'e'   , 0xa711 , 0x1ec3 , 0x0000),
DEADTRANS( L'E'   , 0xa711 , 0x1ec2 , 0x0000),
DEADTRANS( L'o'   , 0xa711 , 0x1ed5 , 0x0000),
DEADTRANS( L'O'   , 0xa711 , 0x1ed4 , 0x0000),

/* Cornu et Accent grave */
DEADTRANS( L' '   , 0xa712 , 0xa712 , 0x0000),
DEADTRANS( L'o'   , 0xa712 , 0x1edd , 0x0000),
DEADTRANS( L'O'   , 0xa712 , 0x1edc , 0x0000),
DEADTRANS( L'u'   , 0xa712 , 0x1eeb , 0x0000),
DEADTRANS( L'U'   , 0xa712 , 0x1eea , 0x0000),

/* Cornu et Accent aigu */
DEADTRANS( L' '   , 0xa713 , 0xa713 , 0x0000),
DEADTRANS( L'o'   , 0xa713 , 0x1edb , 0x0000),
DEADTRANS( L'O'   , 0xa713 , 0x1eda , 0x0000),
DEADTRANS( L'u'   , 0xa713 , 0x1ee9 , 0x0000),
DEADTRANS( L'U'   , 0xa713 , 0x1ee8 , 0x0000),

/* Cornu et Point souscrit */
DEADTRANS( L' '   , 0xa714 , 0xa714 , 0x0000),
DEADTRANS( L'o'   , 0xa714 , 0x1ee3 , 0x0000),
DEADTRANS( L'O'   , 0xa714 , 0x1ee2 , 0x0000),
DEADTRANS( L'u'   , 0xa714 , 0x1ef1 , 0x0000),
DEADTRANS( L'U'   , 0xa714 , 0x1ef0 , 0x0000),

/* Cornu et Tilde */
DEADTRANS( L' '   , 0xa715 , 0xa715 , 0x0000),
DEADTRANS( L'o'   , 0xa715 , 0x1ee1 , 0x0000),
DEADTRANS( L'O'   , 0xa715 , 0x1ee0 , 0x0000),
DEADTRANS( L'u'   , 0xa715 , 0x1eef , 0x0000),
DEADTRANS( L'U'   , 0xa715 , 0x1eee , 0x0000),

/* Cornu et Crochet en chef */
DEADTRANS( L' '   , 0xa716 , 0xa716 , 0x0000),
DEADTRANS( L'o'   , 0xa716 , 0x1edf , 0x0000),
DEADTRANS( L'O'   , 0xa716 , 0x1ede , 0x0000),
DEADTRANS( L'u'   , 0xa716 , 0x1eed , 0x0000),
DEADTRANS( L'U'   , 0xa716 , 0x1eec , 0x0000),

    0, 0
};

static ALLOC_SECTION_LDATA KBDTABLES KbdTables = {
    /*
     * Modifier keys
     */
    &CharModifiers,

    /*
     * Characters tables
     */
    aVkToWcharTable,

    /*
     * Diacritics
     */
    aDeadKey,

    /*
     * Names of Keys
     */
    aKeyNames,
    aKeyNamesExt,
    aKeyNamesDead,

    /*
     * Scan codes to Virtual Keys
     */
    ausVK,
    sizeof(ausVK) / sizeof(ausVK[0]),
    aE0VscToVk,
    aE1VscToVk,

    /*
     * Locale-specific special processing
     */
    MAKELONG(KLLF_ALTGR, KBD_VERSION),

    /*
     * Ligatures
     */
    0,
    0,
    NULL
};

PKBDTABLES KbdLayerDescriptor(VOID)
{
    return &KbdTables;
}
